/* chaine.c
 * Fichier source de l'implantation des fonctions pour les chaines de caracteres
 * Auteur : VOURIOT Thierry
 * Date : 27 octobre 2001
 */

#include "chaine.h"

/* Creation d'une nouvelle chaine de caractere (allocation de la memoire) */
Chaine chnouv() {
	
	Chaine ch = (Chaine) malloc((N+1) * (sizeof(Car)));
	ch[0] = '\0';
	return ch;
}

/* Teste si la chaine est vide */
Bool vide(Chaine ch) {
	
	return ch[0] == '\0';
}

/* Longueur de la chaine */
Nat lgr(Chaine ch) {

	Nat r;
	for (r=0;ch[r]!=0;r++);
	return r;
}

/* Ajoute un caratere en queue de chaine
 * pre aq(ch,c) = lgr(ch)<N
 */
Chaine aq(Chaine ch, Car x) {
	
	Chaine res = chnouv();
	int i;
	
	for (i=0;ch[i]!='\0';i++)
		res[i] = ch[i];
	
	res[i] = x;
	res[i+1] = '\0';
	return res;
}

/* Ajoute un caratere en tete de chaine
 * pre at(ch,c) = lgr(ch)<N
 */
Chaine at(Chaine ch, Car x) {
	
	Chaine res = chnouv();
	int i;
	
	for (i=0;ch[i]!='\0';i++)
		res[i+1] = ch[i];
	
	res[0] = x;
	res[i+1] = '\0';
	return res;
}

/* Tete de la chaine
 * pre t(ch) = !vide(ch)
 */
Car t(Chaine ch) {

	return ch[0];
}

/* Queue de la chaine
 * pre q(ch) = !vide(ch)
 */
Car q(Chaine ch) {

	return ch[lgr(ch)-1];
}

/* Corps de la chaine (chaine sans la tete) */
Chaine co(Chaine ch) {

	Chaine res = chnouv();
	int i;
	
	for (i=1;ch[i]!='\0' && ch[0]!='\0';i++)
		res[i-1] = ch[i];
	
	res[i-1] = '\0';
	return res;
}

/* Corps inverse de la chaine (chaine sans la queue) */
Chaine ci(Chaine ch) {

	Chaine res = chnouv();
	int i;
	
	for (i=0;ch[i]!='\0';i++)
		res[i] = ch[i];
	
	res[i-1] = '\0';
	return res;
}

/* Extraction d'une sous-chaine en fonction d'un indice de depart
 * et d'une longueur
 * pre sch(ch,d,l) = d+l<=lgr(ch)
 */
Chaine sch(Chaine ch,Nat k,Nat l) {

	Chaine res = chnouv();	
	int i;
	for (i=0;l>0;l--) {
		res[i] = ch[k];
		i++;
		k++;
	}
		
	res[i] = '\0';
	return res;
}

/* Concatenation de deux chaines
 * pre conc(ch1,ch2) = lgr(ch1)+lgr(ch2)<=N
 */	
Chaine conc(Chaine ch1, Chaine ch2) {

	Chaine res = chnouv();	
	int i,l;
	
	for (i=0;ch1[i]!='\0';i++)
		res[i] = ch1[i];
	l=i;
	for (i=0;ch2[i]!='\0';i++)
		res[l+i] = ch2[i];
		
	res[l+i] = '\0';
	return res;
}

/* Creation d'une chaine a partir d'un caratere */
Chaine singleton(Car c) {

    Chaine res = chnouv();
    res[0] = c;
    res[1] = '\0';
    return res;
}

/* Lecture d'une chaine au clavier */
Chaine lirech() {
	
	Car c;
	Chaine res = chnouv();	
	
	while ((c=getchar())!='\n')
		res = aq(res,c);
					
	return res;
}

/* Ecriture a l'ecran d'une chaine */
void ecrch(Chaine ch) {

     int i;
     for (i=0;ch[i]!='\0';i++)
         putchar(ch[i]);
}

/* Copie d'une chaine */
Chaine copiech(Chaine ch) {

     Chaine res = chnouv();
     int i;

     for (i=0;ch[i]!='\0';i++)
         res[i] = ch[i];

     res[i] = '\0';
     return res;
}

/* Destruction d'une chaine (liberer la memoire) */
void destch(Chaine ch) {

     free(ch);
}




	
    




