/* chaine.h
 * Fichier en tete pour l'implantation des chaines de carateres
 * Auteur : VOURIOT Thierry
 * Date : 27 octobre 2001
 */

#include "base.h"

/* Nombre de caratere maximum de la chaine */
#define N 256

/* Entiers naturels */
typedef unsigned short Nat;
/* Booleens */
typedef unsigned char Bool;
/* Caratere */
typedef char Car;
/* Chaine de carateres */
typedef Car * Chaine;

/* Creation d'une nouvelle chaine de caractere (allocation de la memoire) */
extern Chaine chnouv(void);

/* Teste si la chaine est vide */
extern Bool vide(Chaine);

/* Ajoute un caratere en tete de chaine
 * pre at(ch,c) = lgr(ch)<N
 */
extern Chaine at(Chaine, Car);

/* Ajoute un caratere en queue de chaine
 * pre aq(ch,c) = lgr(ch)<N
 */
extern Chaine aq(Chaine, Car);

/* Longueur de la chaine */
extern Nat lgr(Chaine);

/* Tete de la chaine
 * pre t(ch) = !vide(ch)
 */
extern Car t(Chaine);

/* Queue de la chaine
 * pre q(ch) = !vide(ch)
 */
extern Car q(Chaine);

/* Corps de la chaine (chaine sans la tete) */
extern Chaine co(Chaine);

/* Corps inverse de la chaine (chaine sans la queue) */
extern Chaine ci(Chaine);

/* Extraction d'une sous-chaine en fonction d'un indice de depart
 * et d'une longueur
 * pre sch(ch,d,l) = d+l<=lgr(ch)
 */
extern Chaine sch(Chaine, Nat, Nat);

/* Concatenation de deux chaines
 * pre conc(ch1,ch2) = lgr(ch1)+lgr(ch2)<=N
 */
extern Chaine conc(Chaine , Chaine);

/* Creation d'une chaine a partir d'un caratere */
extern Chaine singleton(Car);

/* Lecture d'une chaine au clavier */
extern Chaine lirech();

/* Ecriture a l'ecran d'une chaine */
extern void ecrch(Chaine);

/* Copie d'une chaine */
extern Chaine copiech(Chaine);

/* Destruction d'une chaine (liberer la memoire) */
extern void destch(Chaine);
